#include "CHardAttachCamera.h"
#include "CGameObject.h"
//#include "CInputManager.h"
#include "CGameManager.h"
#include "CScriptManager.h"
#include "CInputManager.h"
#include "CGameTime.h"
#include "CCameraManager.h"
#include "CTriggerManager.h"


#define MAX_OFFSET 8.0f
#define MAX_SPEED 250.0f

static D3DXVECTOR3 worldY( 0.0f, 1.0f, 0.0f );

CHardAttachCamera::CHardAttachCamera()
{
	m_CurrentOffset = D3DXVECTOR3(OFFSET_X, OFFSET_Y, OFFSET_Z);
	Init();
}

CHardAttachCamera::CHardAttachCamera(const string& cameraName) : CBasicCamera(cameraName)
{
	m_CurrentOffset = D3DXVECTOR3(OFFSET_X, OFFSET_Y, OFFSET_Z);
	Init();
}

CHardAttachCamera::CHardAttachCamera(const string& cameraName, const D3DXVECTOR3& startPosition) :
CBasicCamera(cameraName, startPosition)
{
	m_CurrentOffset = D3DXVECTOR3(OFFSET_X, OFFSET_Y, OFFSET_Z);
	Init();
}

CHardAttachCamera::CHardAttachCamera(const string& cameraName, const D3DXVECTOR3& startPosition, const D3DXVECTOR3& startLookAt) :
CBasicCamera(cameraName, startPosition, startLookAt)
{
	m_CurrentOffset = D3DXVECTOR3(OFFSET_X, OFFSET_Y, OFFSET_Z);
	Init();
}

CHardAttachCamera::CHardAttachCamera(const string& cameraName, const D3DXVECTOR3& startPosition, CGameObject* startTarget) :
CBasicCamera(cameraName, startPosition, startTarget)
{
	m_CurrentOffset = D3DXVECTOR3(OFFSET_X, OFFSET_Y, OFFSET_Z);
	Init();

	m_mViewMatrix.axis_pos = startTarget->Transform().axis_pos;
}

CHardAttachCamera::~CHardAttachCamera()
{

}

void CHardAttachCamera::Init()
{
	// Set up with default values
	int nNumWaypoints = (int)GameManager().Track().CourseLines().size();

	CScript* triggerDefs = ScriptManager().Script("Trigger Definitions");
	char buffer[128] ="";

	D3DXVECTOR3 offset;
	D3DXVECTOR3 lookat;

	sprintf_s(buffer, sizeof(buffer), "DEFAULT_OFFSET_X");
	offset.x = triggerDefs->ValueAsFloat(buffer);

	sprintf_s(buffer, sizeof(buffer), "DEFAULT_OFFSET_Y");
	offset.y = triggerDefs->ValueAsFloat(buffer);

	sprintf_s(buffer, sizeof(buffer), "DEFAULT_OFFSET_Z");
	offset.z = triggerDefs->ValueAsFloat(buffer);

	sprintf_s(buffer, sizeof(buffer), "DEFAULT_LOOKAT_X");
	lookat.x = triggerDefs->ValueAsFloat(buffer);

	sprintf_s(buffer, sizeof(buffer), "DEFAULT_LOOKAT_Y");
	lookat.y = triggerDefs->ValueAsFloat(buffer);

	sprintf_s(buffer, sizeof(buffer), "DEFAULT_LOOKAT_Z");
	lookat.z = triggerDefs->ValueAsFloat(buffer);

	m_nCurrentWaypoint = GameManager().Player().Waypoint();
	m_nEndWaypoint = m_nCurrentWaypoint + 1;

	m_CurrentOffset = offset;
	m_CurrentLookAt = lookat;

	m_bTransitioning = false;
	m_bBoosting = false;
	m_bTransitionWhenDone = false;

	D3DXVECTOR3 newPosition = m_TargetObject->Transform().axis_pos + (m_TargetObject->Transform().axis_x * m_CurrentOffset.x) + (m_TargetObject->Transform().axis_z * m_CurrentOffset.z) + (m_TargetObject->Transform().axis_y * m_CurrentOffset.y);
	SetPosition(&newPosition);
}

void CHardAttachCamera::Update(void)
{
	float currentSpeed = GameManager().Player().GetCurrentSpeed();
	int currentWaypoint = GameManager().Player().Waypoint();

	float fTick = GameTime().GameTimeStep();

	m_fMu = m_fCurrentMu;

	if (m_bTransitioning)
	{
		// Change for zoom effect when boosting
		if (m_bBoosting)
		{
			m_fCurrentMu += fTick * 3;
		}
		else
		{
			m_fCurrentMu += fTick;
		}

		m_CurrentOffset.x = LinearInterpolate(m_StartOffset.x, m_EndOffset.x, m_fCurrentMu);
		m_CurrentOffset.y = LinearInterpolate(m_StartOffset.y, m_EndOffset.y, m_fCurrentMu);
		m_CurrentOffset.z = LinearInterpolate(m_StartOffset.z, m_EndOffset.z, m_fCurrentMu);

		m_CurrentLookAt.x = LinearInterpolate(m_StartLookAt.x, m_EndLookAt.x, m_fCurrentMu);
		m_CurrentLookAt.y = LinearInterpolate(m_StartLookAt.y, m_EndLookAt.y, m_fCurrentMu);
		m_CurrentLookAt.z = LinearInterpolate(m_StartLookAt.z, m_EndLookAt.z, m_fCurrentMu);

		if (m_fCurrentMu > 1.0f)
		{
			m_bTransitioning = false;
			m_fCurrentMu = 0.0f;

			m_CurrentOffset.x = LinearInterpolate(m_StartOffset.x, m_EndOffset.x, 1.0f);
			m_CurrentOffset.y = LinearInterpolate(m_StartOffset.y, m_EndOffset.y, 1.0f);
			m_CurrentOffset.z = LinearInterpolate(m_StartOffset.z, m_EndOffset.z, 1.0f);

			m_CurrentLookAt.x = LinearInterpolate(m_StartLookAt.x, m_EndLookAt.x, 1.0f);
			m_CurrentLookAt.y = LinearInterpolate(m_StartLookAt.y, m_EndLookAt.y, 1.0f);
			m_CurrentLookAt.z = LinearInterpolate(m_StartLookAt.z, m_EndLookAt.z, 1.0f);
		}
	}

	m_v3Offset = m_CurrentOffset;

	float zOffset = 0;
	float actualSpeed = (250.0f * currentSpeed) / 1.0f;

	zOffset = MAX_OFFSET * currentSpeed;

	D3DXVECTOR3 newPosition;
	newPosition = m_TargetObject->Transform().axis_pos + (m_TargetObject->Transform().axis_x * m_CurrentOffset.x) + (m_TargetObject->Transform().axis_z * (m_CurrentOffset.z - zOffset)) + (m_TargetObject->Transform().axis_y * m_CurrentOffset.y);

	SetPosition(&newPosition);

	m_v3LookAt = m_CurrentLookAt;
	D3DXVECTOR3 newLookAt = m_TargetObject->Transform().axis_pos + (m_TargetObject->Transform().axis_x * m_CurrentLookAt.x) + (m_TargetObject->Transform().axis_z * m_CurrentLookAt.z) + (m_TargetObject->Transform().axis_y * m_CurrentLookAt.y);
	SetLookAtTarget(&newLookAt);
	
	if (m_bIsActive)
	{
		if (D3DXVec3Length(&m_v3Velocity) > m_fMaxVelocity)
		{
			m_v3Velocity = *(D3DXVec3Normalize(&m_v3Velocity, &m_v3Velocity)) * m_fMaxVelocity;
		}

		// Move the Camera by its velocity
		m_v3Position += m_v3Velocity;

		// Could Decelerate
		m_v3Velocity = D3DXVECTOR3(0.0f, 0.0f, 0.0f);
		m_v3Look = m_v3Position + m_v3Look;

		// Calculate new view
		//D3DXVECTOR3 up = D3DXVECTOR3(0.0f, 1.0f, 0.0f);
		D3DXVECTOR3 up = m_TargetObject->Transform().axis_y;
		D3DXMatrixLookAtLH(&m_mViewMatrix.matrix_dx, &m_v3Position, &m_v3Look, & up);

		m_v3Right.x = m_mViewMatrix.matrix_dx._11;
		m_v3Right.y = m_mViewMatrix.matrix_dx._21;
		m_v3Right.z = m_mViewMatrix.matrix_dx._31;
		m_v3Up.x = m_mViewMatrix.matrix_dx._12;
		m_v3Up.y = m_mViewMatrix.matrix_dx._22;
		m_v3Up.z = m_mViewMatrix.matrix_dx._32;
		m_v3Look.x = m_mViewMatrix.matrix_dx._13;
		m_v3Look.y = m_mViewMatrix.matrix_dx._23;
		m_v3Look.z = m_mViewMatrix.matrix_dx._33;

		float lookLengthOnXZ = sqrtf(m_v3Look.z * m_v3Look.z + m_v3Look.x *m_v3Look.x);
		m_fPitch = atan2f(m_v3Look.y, lookLengthOnXZ);
		m_fYaw = atan2f(m_v3Look.x, m_v3Look.z);
	}

	if (((CRacer*)m_TargetObject)->GetIsBoosting() && !m_bBoosting)
	{
		m_SavedOffset = m_CurrentOffset;
		m_SavedLookAt = m_CurrentLookAt;

		D3DXVECTOR3 offset;
		offset.x = 0.0f;
		offset.y = 4.4f;
		offset.z = -30; /*- 8.0f*/
		SwitchCameraPosition(offset, D3DXVECTOR3(TriggerManager().m_fOnBoostRingLookAtX, TriggerManager().m_fOnBoostRingLookAtY, TriggerManager().m_fOnBoostRingLookAtZ));
		m_bBoosting = true;
	}

	if (m_bBoosting && !((CRacer*)m_TargetObject)->GetIsBoosting())
	{
		m_bBoosting = false;
		SwitchCameraPosition(D3DXVECTOR3(TriggerManager().m_fDefaultOffsetX,TriggerManager().m_fDefaultOffsetY, TriggerManager().m_fDefaultOffsetZ),
							 D3DXVECTOR3(TriggerManager().m_fDefaultLookAtX, TriggerManager().m_fDefaultLookAtY, TriggerManager().m_fDefaultLookAtZ));
	}
}

void CHardAttachCamera::SetActive(bool value)
{
	m_bIsActive = value;
}

float CHardAttachCamera::LinearInterpolate(float val1,float val2, float mu)
{
	return(val1*(1-mu)+val2*mu);
}


void CHardAttachCamera::SwitchCameraPosition(D3DXVECTOR3 offset, D3DXVECTOR3 lookAt)
{
	if (m_bBoosting || m_bTransitioning)
	{
		return;
	}

	m_bTransitioning = true;
	m_fCurrentMu = 0.0f;

	m_StartOffset = m_CurrentOffset;
	m_StartLookAt = m_CurrentLookAt;

	m_EndOffset = offset;
	m_EndLookAt = lookAt;
}