#include "CShader.h"

bool CShader::AddVariable(string constantName, D3DXHANDLE* scopeHandle)
{
	if(m_VariableMap.find(constantName) == m_VariableMap.end())
	{
		D3DXHANDLE addHandle = m_pConstantTable->GetConstantByName(0, constantName.c_str());

		if (!addHandle)
			return false;

		m_VariableMap[constantName] = addHandle;

		return true;
	}

	return false;
}

bool CShader::SetBool(string variableName, bool setValue)
{
	if (m_VariableMap.find(variableName) == m_VariableMap.end())
		return false;

	m_pConstantTable->SetBool(m_pDevice, m_VariableMap[variableName], setValue);
	return true;
}

bool CShader::SetBoolArray(string variableName,const bool* setValue, int arraySize)
{
	if (m_VariableMap.find(variableName) == m_VariableMap.end())
		return false;

	m_pConstantTable->SetBoolArray(m_pDevice, m_VariableMap[variableName], (BOOL*)setValue, arraySize);
	return true;
}

bool CShader::SetFloat(string variableName, float setValue)
{
	if (m_VariableMap.find(variableName) == m_VariableMap.end())
		return false;

	m_pConstantTable->SetFloat(m_pDevice, m_VariableMap[variableName], setValue);
	return true;
}

bool CShader::SetFloatArray(string variableName, float* setValue, int arraySize)
{
	if (m_VariableMap.find(variableName) == m_VariableMap.end())
		return false;

	m_pConstantTable->SetFloatArray(m_pDevice, m_VariableMap[variableName], setValue, arraySize);
	return true;
}

bool CShader::SetInt(string variableName, int setValue)
{
	if (m_VariableMap.find(variableName) == m_VariableMap.end())
		return false;

	m_pConstantTable->SetInt(m_pDevice, m_VariableMap[variableName], setValue);
	return true;
}

bool CShader::SetIntArray(string variableName, int* setValue, int arraySize)
{
	if (m_VariableMap.find(variableName) == m_VariableMap.end())
		return false;

	m_pConstantTable->SetIntArray(m_pDevice, m_VariableMap[variableName], setValue, arraySize);
	return true;
}

bool CShader::SetMatrix(string variableName, const D3DXMATRIX* setValue)
{
	if (m_VariableMap.find(variableName) == m_VariableMap.end())
		return false;

	m_pConstantTable->SetMatrix(m_pDevice, m_VariableMap[variableName], setValue);
	return true;
}

bool CShader::SetMatrixArray(string variableName, const D3DXMATRIX* setValue, int arraySize)
{
	if (m_VariableMap.find(variableName) == m_VariableMap.end())
		return false;

	m_pConstantTable->SetMatrixArray(m_pDevice, m_VariableMap[variableName], setValue, arraySize);
	return true;
}

bool CShader::SetMatrixPointerArray(string variableName, const D3DXMATRIX* setValue, int arraySize)
{
	if (m_VariableMap.find(variableName) == m_VariableMap.end())
		return false;

	m_pConstantTable->SetMatrixPointerArray(m_pDevice, m_VariableMap[variableName], &setValue, arraySize);
	return true;
}

bool CShader::SetMatrixTranspose(string variableName, const D3DXMATRIX* setValue)
{
	if (m_VariableMap.find(variableName) == m_VariableMap.end())
		return false;

	m_pConstantTable->SetMatrixTranspose(m_pDevice, m_VariableMap[variableName], setValue);
	return true;
}

bool CShader::SetMatrixTransposeArray(string variableName, const D3DXMATRIX* setValue, int arraySize)
{
	if (m_VariableMap.find(variableName) == m_VariableMap.end())
		return false;

	m_pConstantTable->SetMatrixTransposeArray(m_pDevice, m_VariableMap[variableName], setValue, arraySize);
	return true;
}

bool CShader::SetMatrixTransposePointerArray(string variableName, const D3DXMATRIX* setValue, int arraySize)
{
	if (m_VariableMap.find(variableName) == m_VariableMap.end())
		return false;

	m_pConstantTable->SetMatrixTransposePointerArray(m_pDevice, m_VariableMap[variableName], &setValue, arraySize);
	return true;
}

bool CShader::SetVector(string variableName, D3DXVECTOR4 setValue)
{
	if (m_VariableMap.find(variableName) == m_VariableMap.end())
		return false;

	m_pConstantTable->SetVector(m_pDevice, m_VariableMap[variableName], &setValue);
	return true;
}

bool CShader::SetVectorArray(string variableName, D3DXVECTOR4 setValue, int arraySize)
{
	if (m_VariableMap.find(variableName) == m_VariableMap.end())
		return false;

	m_pConstantTable->SetVectorArray(m_pDevice, m_VariableMap[variableName], &setValue, arraySize);
	return true;
}

bool CShader::SetValue(string variableName, void* structure, int size)
{
	if (m_VariableMap.find(variableName) == m_VariableMap.end())
		return false;

	m_pConstantTable->SetValue(m_pDevice, m_VariableMap[variableName], structure, size);
	return false;
}

void CShader::SetDefaults()
{
	m_pConstantTable->SetDefaults(m_pDevice);
}