#ifndef _CSHADER_H_
#define _CSHADER_H_

// The include files for Direct3D9
#include <d3d9.h>
#include <d3dx9.h>

#include <map>
using std::map;

#include <string>
using std::string;

class CShader
{

protected:

	string						m_szName;
	IDirect3DDevice9* 			m_pDevice;
	ID3DXConstantTable*			m_pConstantTable;
	ID3DXBuffer*				m_pShader;
	ID3DXBuffer*				m_pErrorBuffer;
	string						m_szFileName;
	string						m_szEntryPoint;
	string						m_szVersion;

	map<string, D3DXHANDLE>		m_VariableMap;

private:

	CShader(const CShader &);
	CShader & operator = (const CShader &);

public:

	CShader() {}
	virtual ~CShader() {}

	virtual CShader& Create(string shaderName, string shaderFileName, string shaderEntryPoint, string shaderVersion) = 0;
	virtual bool Release() = 0;

	bool AddVariable(string constantName, D3DXHANDLE* scopeHandle = 0);

	virtual bool SetShader() = 0;
	bool SetBool(string variableName, bool setValue);
	bool SetBoolArray(string variableName, const bool* setValue, int arraySize);
	bool SetFloat(string variableName, float setValue);
	bool SetFloatArray(string variableName, float* setValue, int arraySize);
	bool SetInt(string variableName, int setValue);
	bool SetIntArray(string variableName, int* setValue, int arraySize);
	bool SetMatrix(string variableName, const D3DXMATRIX* setValue);
	bool SetMatrixArray(string variableName, const D3DXMATRIX* setValue, int arraySize);
	bool SetMatrixPointerArray(string variableName, const D3DXMATRIX* setValue, int arraySize);
	bool SetMatrixTranspose(string variableName, const D3DXMATRIX* setValue);
	bool SetMatrixTransposeArray(string variableName, const D3DXMATRIX* setValue, int arraySize);
	bool SetMatrixTransposePointerArray(string variableName, const D3DXMATRIX* setValue, int arraySize);
	bool SetVector(string variableName, D3DXVECTOR4 setValue);
	bool SetVectorArray(string variableName, D3DXVECTOR4 setValue, int arraySize);
	bool SetValue(string variableName, void* structure, int size);
	void SetDefaults();
};

#endif