#include "CTrigger.h"
#include "CGameManager.h"
#include "CHardAttachCamera.h"
#include "CCameraManager.h"
#include "CGameTime.h"
#include "CTriggerManager.h"
#include "CSoundManager.h"

bool cTimerTrigger::Activate()
{
	isActive = true;
	currentTime = 0.0f;
	return true;
}

bool cTimerTrigger::Deactivate()
{
	isActive = false;
	currentTime = 0.0f;
	return true;
}

bool cTimerTrigger::Update()
{
	currentTime += GameTime().GameTimeStep();
	return true;
}

bool cTimerTrigger::Shutdown()
{
	return true;
}

bool cTimerTrigger::ShouldDeactivate()
{
	if (!isActive)
		return false;

	return (currentTime > timeToShutdown);
}

bool cTimerTrigger::ShouldActivate()
{
	if (isActive)
		return false;

	return (GameManager().Player().ActualTrackDistance() > distanceToActivate && GameManager().Player().ActualTrackDistance() < (distanceToActivate + 30.0f));
}

bool cDistanceTrigger::Activate()
{
	isActive = true;
	return true;
}

bool cDistanceTrigger::Deactivate()
{
	isActive = false;
	return true;
}

bool cDistanceTrigger::Update()
{
	return true;
}

bool cDistanceTrigger::Shutdown()
{
	return true;
}

bool cDistanceTrigger::ShouldDeactivate()
{
	if (!isActive)
		return false;

	return (GameManager().Player().ActualTrackDistance() > distanceToShutdown);
}

bool cDistanceTrigger::ShouldActivate()
{
	if (isActive)
		return false;

	return (GameManager().Player().ActualTrackDistance() > distanceToActivate && GameManager().Player().ActualTrackDistance() < distanceToShutdown);
}

bool cCameraPositionChangeTrigger::Activate()
{
	CHardAttachCamera* myCamera = (CHardAttachCamera*)&CameraManager().GetCamera(GameManager().Player().GetCameraName());

	myCamera->SwitchCameraPosition(offset, lookAt);
	return cDistanceTrigger::Activate();
}

bool cCameraPositionChangeTrigger::Deactivate()
{
	isActive = false;

	CHardAttachCamera* myCamera = (CHardAttachCamera*)&CameraManager().GetCamera(GameManager().Player().GetCameraName());

	myCamera->SwitchCameraPosition(D3DXVECTOR3(TriggerManager().m_fDefaultOffsetX, TriggerManager().m_fDefaultOffsetY, TriggerManager().m_fDefaultOffsetZ), D3DXVECTOR3(TriggerManager().m_fDefaultLookAtX, TriggerManager().m_fDefaultLookAtY, TriggerManager().m_fDefaultLookAtZ));

	return cDistanceTrigger::Deactivate();
}


bool cCameraPositionChangeTrigger::Update()
{
	return cDistanceTrigger::Update();
}

bool cCameraPositionChangeTrigger::Shutdown()
{
	return cDistanceTrigger::Shutdown();
}

bool cCameraPositionChangeTrigger::ShouldDeactivate()
{
	if (!isActive)
		return false;

	return cDistanceTrigger::ShouldDeactivate();
}

bool cCameraPositionChangeTrigger::ShouldActivate()
{
	if (isActive)
		return false;

	return cDistanceTrigger::ShouldActivate();
}

bool cBoostRailTrigger::Activate()
{
	isActive = true;

	GameManager().Player().OnRail(true);

	CHardAttachCamera* myCamera = (CHardAttachCamera*)&CameraManager().GetCamera(GameManager().Player().GetCameraName());

	myCamera->SwitchCameraPosition(D3DXVECTOR3(TriggerManager().m_fOnBoostRingOffsetX, TriggerManager().m_fOnBoostRingOffsetY, TriggerManager().m_fOnBoostRingOffsetZ), D3DXVECTOR3(TriggerManager().m_fOnBoostRingLookAtX, TriggerManager().m_fOnBoostRingLookAtY, TriggerManager().m_fOnBoostRingLookAtZ));

	return true;
}

bool cBoostRailTrigger::Deactivate()
{
	isActive = false;

	GameManager().Player().OnRail(false);

	CHardAttachCamera* myCamera = (CHardAttachCamera*)&CameraManager().GetCamera(GameManager().Player().GetCameraName());

	myCamera->SwitchCameraPosition(D3DXVECTOR3(TriggerManager().m_fDefaultOffsetX, TriggerManager().m_fDefaultOffsetY, TriggerManager().m_fDefaultOffsetZ), D3DXVECTOR3(TriggerManager().m_fDefaultLookAtX, TriggerManager().m_fDefaultLookAtY, TriggerManager().m_fDefaultLookAtZ));

	return true;
}

bool cBoostRailTrigger::Update()
{
	return true;
}

bool cBoostRailTrigger::Shutdown()
{
	return true;
}

bool cBoostRailTrigger::ShouldDeactivate()
{
	if (!isActive)
		return false;

	return cDistanceTrigger::ShouldDeactivate();
}

bool cBoostRailTrigger::ShouldActivate()
{
	if (isActive)
		return false;

	return cDistanceTrigger::ShouldActivate();
}

bool cHudSignTrigger::Activate()
{
	GameManager().Player().HUD().DisplayHudSign(signID);
	return cTimerTrigger::Activate();
}

bool cHudSignTrigger::Deactivate()
{
	isActive = false;

	return cTimerTrigger::Deactivate();
}


bool cHudSignTrigger::Update()
{
	return cTimerTrigger::Update();
}

bool cHudSignTrigger::Shutdown()
{
	return cTimerTrigger::Shutdown();
}

bool cHudSignTrigger::ShouldDeactivate()
{
	if (!isActive)
		return false;

	return cTimerTrigger::ShouldDeactivate();
}

bool cHudSignTrigger::ShouldActivate()
{
	if (isActive)
		return false;

	return cTimerTrigger::ShouldActivate();
}

bool cRailApproachTrigger::Activate()
{
	return cTimerTrigger::Activate();
}

bool cRailApproachTrigger::Deactivate()
{
	isActive = false;

	return cTimerTrigger::Deactivate();
}


bool cRailApproachTrigger::Update()
{
	return cTimerTrigger::Update();
}

bool cRailApproachTrigger::Shutdown()
{
	return cTimerTrigger::Shutdown();
}

bool cRailApproachTrigger::ShouldDeactivate()
{
	if (!isActive)
		return false;

	return cTimerTrigger::ShouldDeactivate();
}

bool cRailApproachTrigger::ShouldActivate()
{
	if (isActive)
		return false;

	return cTimerTrigger::ShouldActivate();
}

bool cCheckpointTrigger::Activate()
{
	updateCurrentSound = true;

	char buffer[128] ="";

	sprintf_s(buffer, sizeof(buffer), "BS_Asteroid%i", id);

	SoundManager().PlayStreaming(buffer);

	return cTimerTrigger::Activate();
}

bool cCheckpointTrigger::Deactivate()
{
	isActive = false;
	updateCurrentSound = false;

	volumeToGoBackTo = SoundManager().GetMusicVolume();

	return cTimerTrigger::Deactivate();
}


bool cCheckpointTrigger::Update()
{
	return cTimerTrigger::Update();
}

bool cCheckpointTrigger::Shutdown()
{
	return cTimerTrigger::Shutdown();
}

bool cCheckpointTrigger::ShouldDeactivate()
{
	if (!isActive)
		return false;

	return cTimerTrigger::ShouldDeactivate();
}

bool cCheckpointTrigger::ShouldActivate()
{
	if (isActive)
		return false;

	return cTimerTrigger::ShouldActivate();
}